@@ -178,44 +178,148 @@ public:
   }
 };
 
+class EntryObject : public Lum::Object
+{
+private:
+  Lum::OS::FontRef font;
+  const Entry      *entry;
+  size_t           dateWidth;
+
+public:
+  EntryObject()
+  : font(Lum::OS::display->GetFont())
+  {
+    dateWidth=0;
+    for (size_t i=0; i<=9; i++) {
+      dateWidth=std::max(dateWidth,font->StringWidth(Lum::Base::NumberToWString(i)));
+    }
+
+    dateWidth=4*dateWidth+font->StringWidth(L":");
+  }
+
+  void SetEntry(const Entry* entry)
+  {
+    this->entry=entry;
+  }
+
+  void CalcSize()
+  {
+    minWidth=0;
+    minHeight=font->height;
+
+    width=minWidth;
+    height=minHeight;
+
+    Object::CalcSize();
+  }
+
+  void Draw(int x, int y, size_t w, size_t h)
+  {
+    Object::Draw(x,y,w,h);
+
+    if (!OIntersect(x,y,w,h)) {
+      return;
+    }
+
+    /* --- */
+
+    Lum::OS::DrawInfo *draw=GetDrawInfo();
+
+    draw->PushFont(font);
+    draw->PushForeground(parent->GetTextColor(draw));
+
+    //
+    // Date of entry
+    //
+
+    std::wstring date;
+
+    date=Lum::Base::NumberToWString(entry->time.GetHour());
+    date.append(L":");
+    if (entry->time.GetMinute()<10) {
+      date.append(L"0");
+    }
+    else {
+      date.append(Lum::Base::NumberToWString(entry->time.GetMinute()/10));
+    }
+    date.append(Lum::Base::NumberToWString(entry->time.GetMinute()%10));
+
+
+    draw->DrawString(this->x,
+                     this->y+font->ascent,
+                     date);
+
+    //
+    // Name of entry
+    //
+
+    std::wstring name;
+
+    if (entry->item.empty()) {
+      name=_(L"LABEL_WEIGHT",L"Weight");
+    }
+    else {
+      name=entry->item+L" ("+AmountToString(entry->amount)+L"x"+entry->unit+L")";
+    }
+
+    draw->DrawString(this->x+
+                     dateWidth+
+                     Lum::OS::display->GetSpaceHorizontal(Lum::OS::Display::spaceObjectBorder),
+                     this->y+font->ascent,
+                     name);
+
+    //
+    // Value + unit
+    //
+
+    std::wstring value;
+
+    if (entry->item.empty()) {
+      value=AmountToString(entry->amount)+L" "+weightUnit->Get();
+    }
+    else {
+      value=ValueToString((((Value)entry->amount)*entry->value)/10)+L" "+valueUnit->Get();
+    }
+
+    draw->DrawString(this->x+width-font->StringWidth(value),
+                     this->y+font->ascent,
+                     value);
+
+    draw->PopFont();
+    draw->PopForeground();
+  }
+
+  static EntryObject* Create(bool horizFlex, bool vertFlex)
+  {
+    EntryObject *object=new EntryObject();
+
+    object->SetFlex(horizFlex,vertFlex);
+
+    return object;
+  }
+};
+
 class EntryDataProvider : public EntryModel::DataProvider
 {
+private:
+  EntryObject *object;
+
 public:
-  std::wstring GetString(const EntryModel::Iterator& iter, size_t column) const
+  EntryDataProvider()
+  : object(EntryObject::Create(true,false))
   {
-    if (column==1) {
-      std::wstring res;
+    // no code
+  }
 
-      res=Lum::Base::NumberToWString((*iter)->time.GetHour());
-      res.append(L":");
-      if ((*iter)->time.GetMinute()<10) {
-        res.append(L"0");
-      }
-      else {
-        res.append(Lum::Base::NumberToWString((*iter)->time.GetMinute()/10));
-      }
-      res.append(Lum::Base::NumberToWString((*iter)->time.GetMinute()%10));
+  ~EntryDataProvider()
+  {
+    delete object;
+  }
 
-      return res;
-    }
-    else if (column==2) {
-      if ((*iter)->item.empty()) {
-        return _(L"LABEL_WEIGHT",L"Weight");
-      }
-      else {
-        return (*iter)->item+L" ("+AmountToString((*iter)->amount)+L" "+(*iter)->unit+L")";
-      }
-    }
-    else if (column==3) {
-      if ((*iter)->item.empty()) {
-        return AmountToString((*iter)->amount)+L" "+weightUnit->Get();
-      }
-      else {
-        return ValueToString((((Value)(*iter)->amount)*(*iter)->value)/10)+L" "+valueUnit->Get();
-      }
-    }
-
-    assert(false);
+  Lum::Object* GetObject(const EntryModel::Iterator& iter, size_t column) const
+  {
+    object->SetEntry(*iter);
+    return object;
   }
 };
 
@@ -275,7 +379,7 @@ void DataInput::CalcSize()
   Lum::DateSelect       *dateSelect;
   Lum::Menu             *menu;
   Lum::Panel            *et;
-  Lum::Panel            *ds;
+  Lum::Panel            *toolbar;
   Lum::Panel            *horiz;
   Lum::Panel            *vert;
   Lum::Table            *table;
@@ -285,35 +389,35 @@ void DataInput::CalcSize()
 
   // Date select
 
-  ds=Lum::HPanel::Create(true,false);
-  ds->Add(Lum::Button::Create(L"_<",prevDayAction));
-  ds->Add(new Lum::HSpace(Lum::Space::sizeSmall));
+  toolbar=Lum::HPanel::Create(true,false);
+  toolbar->Add(Lum::Button::Create(L"_<",prevDayAction));
+  toolbar->Add(new Lum::HSpace(Lum::Space::sizeSmall));
 
   dateSelect=new Lum::DateSelect();
   dateSelect->SetModel(date);
-  ds->Add(dateSelect);
+  toolbar->Add(dateSelect);
 
-  ds->Add(new Lum::HSpace(Lum::Space::sizeSmall));
-  ds->Add(Lum::Button::Create(L"_>",nextDayAction));
-  ds->AddSpace();
+  toolbar->Add(new Lum::HSpace(Lum::Space::sizeSmall));
+  toolbar->Add(Lum::Button::Create(L"_>",nextDayAction));
+  toolbar->AddSpace();
 
   trend=new Trend();
   trend->SetModel(this->trend);
-  ds->Add(trend);
+  toolbar->Add(trend);
 
-  ds->AddSpace();
+  toolbar->AddSpace();
 
   text=Lum::TextValue::Create(averageDiff,Lum::TextValue::left,true,false);
   text->SetMinWidth(Lum::Base::Size::stdCharWidth,5);
-  ds->Add(text);
+  toolbar->Add(text);
 
-  ds->AddSpace();
+  toolbar->AddSpace();
 
   text=Lum::TextValue::Create(summary,Lum::TextValue::right,true,false);
   text->SetMinWidth(Lum::Base::Size::stdCharWidth,8);
-  ds->Add(text);
+  toolbar->Add(text);
 
-  ds->AddSpace(true)
+  toolbar->AddSpace(true)
     ->Add(Lum::Button::Create(L"+",
                               Lum::OS::Theme::imageAdd,
                               addWeightAction,
@@ -329,9 +433,7 @@ void DataInput::CalcSize()
   et=Lum::HPanel::Create(true,true);
 
   headerModel=new Lum::Model::HeaderImpl();
-  headerModel->AddColumn(_(L"ENTRIES_TABLE_HEADER_TIME",L"Time"),Lum::Base::Size::stdCharWidth,5);
-  headerModel->AddColumn(_(L"ENTRIES_TABLE_HEADER_ITEM",L"Item"),Lum::Base::Size::stdCharWidth,15,true);
-  headerModel->AddColumn(_(L"ENTRIES_TABLE_HEADER_VALUE",L"Value"),Lum::Base::Size::stdCharWidth,8);
+  headerModel->AddColumn(L"Entry",Lum::Base::Size::stdCharWidth,30,true);
 
   table=new Lum::Table();
   table->SetFlex(true,true);
@@ -339,7 +441,7 @@ void DataInput::CalcSize()
   table->SetModel(entries);
   table->SetSelection(entrySelection);
   table->SetDoubleClickAction(editItemAction);
-  table->SetShowHeader(true);
+  table->SetShowHeader(false);
   table->GetTableView()->SetAutoFitColumns(true);
   table->GetTableView()->SetAutoHSize(true);
   et->Add(table);
@@ -357,9 +459,18 @@ void DataInput::CalcSize()
 
   vert=Lum::VPanel::Create(true,true);
 
-  vert->Add(ds);
-  vert->AddSpace();
-  vert->Add(et);
+  switch (Lum::OS::display->GetTheme()->GetToolbarPosition()) {
+    case Lum::OS::Theme::toolbarBottom:
+    vert->Add(et);
+    vert->AddSpace();
+    vert->Add(toolbar);
+    break;
+  default:
+    vert->Add(toolbar);
+    vert->AddSpace();
+    vert->Add(et);
+    break;
+  }
 
   horiz->Add(vert);
 
