@@ -188,19 +188,103 @@ public:
   }
 };
 
+class FoodObject : public Lum::Object
+{
+private:
+  Lum::OS::FontRef font;
+  const Food       *food;
+
+public:
+  FoodObject()
+  : font(Lum::OS::display->GetFont())
+  {
+    // no code
+  }
+
+  void SetFood(const Food* food)
+  {
+    this->food=food;
+  }
+
+  void CalcSize()
+  {
+    minWidth=0;
+    minHeight=font->height;
+
+    width=minWidth;
+    height=minHeight;
+
+    Object::CalcSize();
+  }
+
+  void Draw(int x, int y, size_t w, size_t h)
+  {
+    Object::Draw(x,y,w,h);
+
+    if (!OIntersect(x,y,w,h)) {
+      return;
+    }
+
+    /* --- */
+
+    Lum::OS::DrawInfo *draw=GetDrawInfo();
+
+    draw->PushFont(font);
+    draw->PushForeground(parent->GetTextColor(draw));
+
+    //
+    // Name of food
+    //
+
+    draw->DrawString(this->x,
+                     this->y+font->ascent,
+                     food->name+L" ("+food->unit+L")");
+
+    //
+    // Value + unit
+    //
+
+    std::wstring value=ValueToString(food->value)+L" "+valueUnit->Get();
+
+    draw->DrawString(this->x+width-font->StringWidth(value),
+                     this->y+font->ascent,
+                     value);
+
+    draw->PopFont();
+    draw->PopForeground();
+  }
+
+  static FoodObject* Create(bool horizFlex, bool vertFlex)
+  {
+    FoodObject *object=new FoodObject();
+
+    object->SetFlex(horizFlex,vertFlex);
+
+    return object;
+  }
+};
+
 class FoodDataProvider : public FoodModel::DataProvider
 {
+private:
+  FoodObject *object;
+
 public:
-  std::wstring GetString(const FoodModel::Iterator& iter, size_t column) const
+  FoodDataProvider()
+  : object(FoodObject::Create(true,false))
   {
-    switch (column) {
-    case 1:
-      return (*iter)->name+L" ("+(*iter)->unit+L")";
-    case 2:
-      return ValueToString((*iter)->value)+L" "+valueUnit->Get();
-    default:
-      return L"";
-    }
+    // no code
+  }
+
+  ~FoodDataProvider()
+  {
+    delete object;
+  }
+
+  Lum::Object* GetObject(const FoodModel::Iterator& iter, size_t column) const
+  {
+    object->SetFood(*iter);
+    return object;
   }
 };
 
@@ -291,7 +375,7 @@ void FoodInput::CalcSize()
 {
   Lum::Menu             *menu;
   Lum::Panel            *ft;
-  Lum::Panel            *sb;
+  Lum::Panel            *toolbar;
   Lum::Panel            *horiz;
   Lum::Panel            *vert;
   Lum::Table            *table;
@@ -299,8 +383,8 @@ void FoodInput::CalcSize()
 
   // Search box
 
-  sb=Lum::VPanel::Create(true,false);
-  sb->Add(Lum::HPanel::Create(true,false)
+  toolbar=Lum::VPanel::Create(true,false);
+  toolbar->Add(Lum::HPanel::Create(true,false)
           ->Add(Lum::Label::Create(true,false)
                 ->AddLabel(L"Search:",
                            Lum::String::Create(foodSearch,true,false)
@@ -333,8 +417,7 @@ void FoodInput::CalcSize()
   ft=Lum::HPanel::Create(true,true);
 
   headerModel=new Lum::Model::HeaderImpl();
-  headerModel->AddColumn(_(L"FOOD_TABLE_HEADER_FOOD",L"Food"),Lum::Base::Size::stdCharWidth,20,true);
-  headerModel->AddColumn(_(L"FOOD_TABLE_HEADER_VALUE",L"Value"),Lum::Base::Size::stdCharWidth,8);
+  headerModel->AddColumn(L"Food",Lum::Base::Size::stdCharWidth,30,true);
 
   table=new Lum::Table();
   table->SetFlex(true,true);
@@ -342,7 +425,7 @@ void FoodInput::CalcSize()
   table->SetModel(foods);
   table->SetSelection(foodSelection);
   table->SetDoubleClickAction(registerFoodAction);
-  table->SetShowHeader(true);
+  table->SetShowHeader(false);
   table->GetTableView()->SetAutoFitColumns(true);
   table->GetTableView()->SetAutoHSize(true);
   table->GetTableView()->SetFormatProvider(new FoodFormatProvider(foods));
@@ -361,9 +444,18 @@ void FoodInput::CalcSize()
 
   vert=Lum::VPanel::Create(true,true);
 
-  vert->Add(sb);
-  vert->AddSpace();
-  vert->Add(ft);
+  switch (Lum::OS::display->GetTheme()->GetToolbarPosition()) {
+    case Lum::OS::Theme::toolbarBottom:
+    vert->Add(ft);
+    vert->AddSpace();
+    vert->Add(toolbar);
+    break;
+  default:
+    vert->Add(toolbar);
+    vert->AddSpace();
+    vert->Add(ft);
+    break;
+  }
 
   horiz->Add(vert);
 
